// =============================================================
// File:	hrtools.js
// Purpose:	some simple tools
// Copyright:	(c) 2018 Henk Reints, MSc. http://henk-reints.nl
// Date:	2018-06-19
// =============================================================
function rpad(x,w,p)
{	y = arguments.length > 2 ? ""+p : "";
	if (y.length == 0) y = " ";
	for (var r = ""+x; r.length < w; r += y);
	return r;
};
function lpad(x,w,p)
{	y = arguments.length > 2 ? ""+p : "";
	if (y.length == 0) y = " ";
	for (var r = ""+x; r.length < w; r = y+r);
	return r;
};
function toNumber(input)
{	return isNaN(parseFloat(input)) ? NaN : Number(input);
};
var decimalPrefixList = "yzafpnum kMGTPEZY";
var decimalPrefixOffset = 8;
var decimalFactorExponent = 3;
//
function formatNumber(x,precision,unitSymbol,integerWidth,mantissaWidth,fullWidth,totalWidth)
{	var digitCount = toNumber(precision), maxPrecision = 21, maxFractionDigits = maxPrecision-1;
	var doFixed = digitCount < 1;
	var result = "";
	if (doFixed)
	{	digitCount = -digitCount;
		if (digitCount > maxFractionDigits) {digitCount = NaN, doFixed = false};
	} else
	{	if (digitCount > maxPrecision) digitCount = NaN;
	};
	var unit0 = arguments.length	> 2	? ""+unitSymbol	: "";
	var unit1 = unit0		!= ""	? " "+unit0	: "";
	if (!isFinite(x))
	{	result = rpad(lpad(lpad(""+x,mantissaWidth),fullWidth),totalWidth);

	} else if (doFixed)
	{	result = x.toFixed(digitCount);
		if (result.indexOf(".") < 0) result += ".";
		while (result.indexOf(".") < integerWidth) result = " "+result;
		result = result.replace(/\.$/,"");
		result = rpad(result,mantissaWidth)+unit1;
	} else
	{	var sign = (x < 0 ? "-" : ""), xAbs = Math.abs(x);
		var xInfo = (isFinite(digitCount) ? xAbs.toExponential(digitCount-1) : xAbs.toExponential()).split("e");
		var mantissaParts = xInfo[0].split(".");
		var mantissaInteger  = mantissaParts[0];
		var mantissaFraction = mantissaParts.length > 1 ? mantissaParts[1] : "";
		var exponent = toNumber(xInfo[1]);
		var exponentOverflow = (exponent%decimalFactorExponent+decimalFactorExponent)%decimalFactorExponent;
		if (exponentOverflow > 0)
		{	var temp = mantissaFraction.slice(0,exponentOverflow);
			mantissaFraction = mantissaFraction.slice(temp.length);
			while (temp.length < exponentOverflow) temp += "0";
			mantissaInteger += temp;
			exponent -= exponentOverflow;
		};
		result = lpad(sign+mantissaInteger,integerWidth);
		if (mantissaFraction != "") result += "."+mantissaFraction;
		result = rpad(result,mantissaWidth);
		var decimalPrefixIndex = decimalPrefixOffset + Math.floor(exponent/decimalFactorExponent);
		var decimalPrefix = decimalPrefixList.charAt(decimalPrefixIndex);
		if (decimalPrefix == "" && exponent != 0)
		{	result += "e"+exponent;
			if (unit0 != "") decimalPrefix = " ";
		};
		result = rpad(result,fullWidth);
		result += (unit0 != "" ? " "+decimalPrefix+unit0 : decimalPrefix).replace(/\s+$/,"");
	};
	return rpad(result,totalWidth);
};
// =====================================================================================
// End of file - hrtools.js - Copyright (c) 2018 Henk Reints, MSc. http://henk-reints.nl
